cellam ELLAM added
C     ******************************************************************
C     MAIN CODE FOR U.S. GEOLOGICAL SURVEY MODULAR MODEL -- MODFLOW-96
C           BY MICHAEL G. MCDONALD AND ARLEN W. HARBAUGH
C     MODFLOW-88 documented in:
C           McDonald, M.G. and Harbaugh, A.W., 1988, A modular
C           three-dimensional finite-difference ground-water flow
C           model: U.S. Geological Survey Techniques of Water
C           Resources Investigations, Book 6, Chapter A1, 586 p.
C     MODFLOW-96 documented in:
C           Harbaugh, A.W. and McDonald, M.G., 1996, User's
C           documentation for the U.S. Geological Survey modular
C           finite-difference ground-water flow model: U.S. Geological
C           Survey Open-File Report 96-485
C-----VERSION 0950 23MAY1996 MAIN
C-----VERSION 1401 03DEC1996 -- added PCG2, HFB1, and DE45
C                               as documented
C                               in USGS reports
C     MOC3D Version 1.0 11/08/96
C     Documented in:
C           Konikow, L.F., Goode, D.J., and Hornberger, G.Z., 1996,
C           A Three-Dimensional Method-of-Characteristics Solute-
C           Transport Model (MOC3D): U.S. Geological Survey Water-
C           Resources Investigations Report 96-4267, 87 p.
C     MOC3D Version 1.1 05/14/97
C     MOC3D Version 1.2 05/01/98
C     MOC3D Version 2.0 11/16/98
C     Documented in:
C           Kipp, K.L., Konikow, L.F., and Hornberger, G.Z., 1998,
C           An Implicit Dispersive Transport Algorithm for
C           the MOC3D Solute-Transport Model: U.S. Geological Survey
C           Water-Resources Investigations Report 98-4234, 54 p.
C     MOC3D VERSION 3.0 1999/03/24
C     Documented in:
C          Goode, D.J., 1999, Age, Double Porosity, and Simple Reaction
C          Modifications for MOC3D Ground-Water Transport Model:
C          U.S. Geological Survey Water-Resources Investigations
C          Report 99-4041, 34 p.
C     MOC3D VERSION 3.5 July 2000 
C     Documented in:
C          Heberton, C.I., Russel, T.F., Konikow, L.F., Hornberger, 
C          G.Z., 2000, A Three-Dimensional Finite Volume Eulerian-
C          Lagrangian Localized Adjoint Method (ELLAM) for Solute-
C          Transport Modeling: U.S. Geological Survey Water-Resources 
C          Investigations Report 00-4087, 63 p.
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C1------SPECIFY THE SIZE OF THE X ARRAY.  TO CHANGE THE SIZE OF THE
C1------X ARRAY, CHANGE VALUE OF LENX IN THE NEXT STATEMENT.
cellam
cgzh burnett3d      PARAMETER  (LENX=25046050)
      PARAMETER  (LENX=40000000)
      PARAMETER (LENMX=10000000)
      common /iint/mx(lenmx)
cellam
      COMMON X(LENX)
      COMMON /FLWCOM/LAYCON(200)
CMOC
      COMMON /MOC3D/ CDEL,RDEL,CNOFLO,CELDIS,FZERO,NZCRIT
      COMMON /SUBGRD/
     *  ISCOL1,ISCOL2,ISROW1,ISROW2,ISLAY1,ISLAY2,ISUBGD
      COMMON/CAIS/MAR(6,6),MAR1(19,19)
cellam
      COMMON /ELLAM/ CINV,RINV,BINV,HCINV,HRINV,HBINV,
     *               STINIT,ADINIT,STMASS,ADMASS,OLMASS,
     *               AZERO,
     *               NSC,NSR,NSL,NT,NCTF,NRTF,NLTF,
     *               NEIGHB(8,2),NSLOPE(3),
     *               IDTOP,IDMAX,NCOEF,NSCH,NSRH,NSLH
cellam
      CHARACTER*16 VBNM(40)
CMOC
      DIMENSION SBVL(4,40),JUNIT(40)
      CHARACTER*80 HEADNG(2)
      DIMENSION VBVL(4,40),IUNIT(40)
      DOUBLE PRECISION DECAY
      DOUBLE PRECISION DUMMY, DUM
      EQUIVALENCE (DUMMY,X(1))
      CHARACTER*20 CHEDFM,CDDNFM
      CHARACTER*80 FNAME
      LOGICAL EXISTS
      CHARACTER*4 CUNIT(40)
CMOC
      CHARACTER*4 DUNIT(40)
      DATA CUNIT/'BCF ','WEL ','DRN ','RIV ','EVT ','    ','GHB ',
     1           'RCH ','SIP ','DE4 ','SOR ','OC  ','PCG ','    ',
     2           'CONC','HFB ','    ','    ','    ','    ','FHB ',
     3           '    ','    ','    ','    ','    ','    ','    ',
     4           '    ','    ','    ','    ','    ','    ','    ',
     5           '    ','    ','    ','    ','    '/
C     ------------------------------------------------------------------
CMOC
      DATA DUNIT/'CRCH','CNCA','CNCB','PRTA','PRTB','VELA','VELB',
     1           'OBS ','AGE ','DP  ','DK  ','    ','    ','    ',
     2           '    ','    ','    ','    ','    ','    ','    ',
     3           '    ','    ','    ','    ','    ','    ','    ',
     4           '    ','    ','    ','    ','    ','    ','    ',
     5           '    ','    ','    ','    ','    '/
C     set string for use if RCS ident command
      FNAME =
     &'$Id: main.f,v 1.4 1997/05/12 20:13:17 rsregan Exp rsregan $'
      FNAME =
     &'@(#)MOC3D - 3-D Method-of-Characteristics Solute-Transport Model'
      FNAME =
     &   '@(#)MOC3D - USGS WRIR 96-4267, Konikow, Goode, and Hornberger'
      FNAME =
     &  '@(#)MOC3D - USGS TWRI, Book 6, Chap. A1, McDonald and Harbaugh'
      FNAME = '@(#)MOC3D - USGS OFR 96-485, Harbaugh and McDonald'
      FNAME = '@(#)MOC3D - Contact: h2osoft@usgs.gov'
      FNAME = '@(#)MOC3D - Version: 3.5 July 2000'
C     ------------------------------------------------------------------
        INUNIT=99
      IBUNIT=98
      IBOUTS=97
      IBATCH=0
      DUM=0.0D0
      INQUIRE(FILE='modflow.bf',EXIST=EXISTS)
      IF(EXISTS) THEN
         IBATCH=1
         OPEN(UNIT=IBUNIT,FILE='modflow.bf',STATUS='OLD')
         OPEN(UNIT=IBOUTS,FILE='modbatch.rpt')
         WRITE(IBOUTS,*) ' USGS MODFLOW MODEL BATCH-MODE REPORT'
      END IF
C
C2------OPEN FILE OF FILE NAMES.
50    IF(IBATCH.GT.0) THEN
         READ(IBUNIT,'(A)',END=500) FNAME
         IF(FNAME.EQ.' ') GO TO 50
         WRITE(IBOUTS,'(1X,/1X,A)') FNAME
      ELSE
         WRITE(*,*) ' Enter the name of the NAME FILE:'
         READ(*,'(A)') FNAME
      END IF
      INQUIRE(FILE=FNAME,EXIST=EXISTS)
      IF(.NOT.EXISTS) THEN
         IF(IBATCH.GT.0) THEN
            WRITE(IBOUTS,*) ' Specified name file does not exist.'
            WRITE(IBOUTS,*) ' Processing will continue with the next ',
     1                      'name file in modflow.bf.'
         ELSE
            WRITE(*,*) ' File does not exist'
         END IF
         GO TO 50
      END IF
      OPEN(UNIT=INUNIT,FILE=FNAME,STATUS='OLD')
C
C3------DEFINE PROBLEM--ROWS,COLUMNS,LAYERS,STRESS PERIODS,PACKAGES.
      CALL BAS5DF(ISUM,HEADNG,NPER,ITMUNI,TOTIM,NCOL,NROW,NLAY,
     1        NODES,INBAS,IOUT,IUNIT,CUNIT,INUNIT,IXSEC,ICHFLG,IFREFM)
C
CMOC----DEFINE MOC SOLUTE-TRANSPORT PROBLEM, SUBGRID, ETC.
      IF(IUNIT(15).GT.0) 
     *   CALL MOC6DF(NCOL,NROW,NLAY,IOUT,IOUTS,IUNIT(15),
     *   INMOC,JUNIT, DUNIT,
     *   NSCOL,NSROW,NSLAY,NODESS,NPMAX,NLIMBO,
     *   NEWPTS,NUMOBS,LSOBSW,LSOBTE,
     *  ICSTRT,ICONLY,NODISP,DECAY,DIFFUS,NCINFL,
     *  IABOVE,IBELOW,
     *  IDIM,NPTPND,MOCTYPE,IC1,IC2,IR1,IR2,IL1,IL2,
     *  IDKTIM,IDKRF,IDKZO,IDKFO,IDKZS,IDKFS,
     *  AGER8,
     *  IDPZO,IDPFO,IDPTIM,IDPPS)
C
CMOC----READ NUMBER OF OBSERVATIONS
C
      IF(IUNIT(15).GT.0.AND.JUNIT(8).GT.0)
     *   CALL OBS5DF(NUMOBS,IOUTS,JUNIT(8),IOBSFL,NPNTOB)
C
C4------ALLOCATE SPACE IN "X" ARRAY.
      CALL BAS5AL(ISUM,LENX,LCHNEW,LCHOLD,LCIBOU,LCCR,LCCC,LCCV,
     1              LCHCOF,LCRHS,LCDELR,LCDELC,LCSTRT,LCBUFF,LCIOFL,
     2              INBAS,ISTRT,NCOL,NROW,NLAY,IOUT,IAPART,IFREFM)
      IF(IUNIT(1).GT.0) CALL BCF5AL(ISUM,LENX,LCSC1,LCHY,
     1     LCBOT,LCTOP,LCSC2,LCTRPY,IUNIT(1),ISS,
     2     NCOL,NROW,NLAY,IOUT,IBCFCB,LCWETD,IWDFLG,LCCVWD,
     3     WETFCT,IWETIT,IHDWET,HDRY,IAPART,IFREFM)
      IF(IUNIT(2).GT.0) CALL WEL5AL(ISUM,LENX,LCWELL,MXWELL,NWELLS,
     1                 IUNIT(2),IOUT,IWELCB,NWELVL,IWELAL,IFREFM)
      IF(IUNIT(3).GT.0) CALL DRN5AL(ISUM,LENX,LCDRAI,NDRAIN,MXDRN,
     1                 IUNIT(3),IOUT,IDRNCB,NDRNVL,IDRNAL,IFREFM)
      IF(IUNIT(4).GT.0) CALL RIV5AL(ISUM,LENX,LCRIVR,MXRIVR,NRIVER,
     1            IUNIT(4),IOUT,IRIVCB,NRIVVL,IRIVAL,IFREFM)
      IF(IUNIT(5).GT.0) CALL EVT5AL(ISUM,LENX,LCIEVT,LCEVTR,LCEXDP,
     1            LCSURF,NCOL,NROW,NEVTOP,IUNIT(5),IOUT,IEVTCB,IFREFM)
      IF(IUNIT(7).GT.0) CALL GHB5AL(ISUM,LENX,LCBNDS,NBOUND,MXBND,
     1            IUNIT(7),IOUT,IGHBCB,NGHBVL,IGHBAL,IFREFM)
      IF(IUNIT(8).GT.0) CALL RCH5AL(ISUM,LENX,LCIRCH,LCRECH,NRCHOP,
     1            NCOL,NROW,IUNIT(8),IOUT,IRCHCB,IFREFM)
      IF(IUNIT(9).GT.0) CALL SIP5AL(ISUM,LENX,LCEL,LCFL,LCGL,LCV,
     1          LCHDCG,LCLRCH,LCW,MXITER,NPARM,NCOL,NROW,NLAY,
     2          IUNIT(9),IOUT,IFREFM)
      IF(IUNIT(10).GT.0) CALL DE45AL(ISUM,LENX,LCAU,LCAL,LCIUPP,
     1           LCIEQP,LCD4B,LCLRCH,LCHDCG,
     2           MXUP,MXLOW,MXEQ,MXBW,IUNIT(10),ITMX,ID4DIR,
     3           NCOL,NROW,NLAY,IOUT,ID4DIM)
      IF(IUNIT(11).GT.0) CALL SOR5AL(ISUM,LENX,LCA,LCRES,LCHDCG,LCLRCH,
     1       LCIEQP,MXITER,NCOL,NLAY,NSLICE,MBW,IUNIT(11),IOUT,IFREFM)
      IF(IUNIT(13).GT.0) CALL PCG2AL(ISUM,LENX,LCV,LCSS,LCP,LCCD,
     1       LCHCHG,LCLHCH,LCRCHG,LCLRCH,MXITER,ITER1,NCOL,NROW,NLAY,
     2       IUNIT(13),IOUT,NPCOND,LCIT1)
      IF(IUNIT(16).GT.0) CALL HFB1AL(ISUM,LENX,LCHFBR,NHFB,IUNIT(16),     *HFB*
     1           IOUT)                                                    *HFB*
      IF(IUNIT(21).GT.0) CALL FHB1AL(ISUM,LENX,LCFLLC,LCBDTM,LCFLRT,
     1          LCBDFV,LCBDHV,LCHDLC,LCSBHD,NBDTIM,NFLW,NHED,IUNIT(21),
     2          IOUT,IFHBCB,NFHBX1,NFHBX2,IFHBD3,IFHBD4,IFHBD5,
     3          IFHBSS,ISS)
C
CMOC----ALLOCATE SPACE IN "X" ARRAY FOR TRANSPORT
      IF(IUNIT(15).GT.0) CALL MOC6AL(ISUM,LENX,
cellam
     *   NISUM,LENMX,LSCB,LSLB,
     *   LSIDB,LSAS,LSIAS,LSJAS,LSA,LSIA,LSJA,LSIW,LSRW,
     *   LSRHSE,LSRHSO,LSDO,LSNONU,LSSAV,LSXFOR,LSXBAC,LSYFOR,LSYBAC,
     *   LSCFOR,LSRFOR,LSTFOR,LSCONL,LSVOL,LSIACT,LSNZIN,
     *   LSDIST,LSDIDS,NTFACE,LENW,LENIW,LSIBOU,
cellam
     *   LSDCC,LSDCR,LSDCL,LSDRR,LSDRC,LSDRL,LSDLL,LSDLC,LSDLR,
     *   LSPC,LSPR,LSPL,LSPCON,LSPTID,LSLMBO,LSIGNP,
     *   LSRF,LSCONC,LSCINT,LSCHBC,LSCINF,
     *   LSSRCS,LSSRCF,LSSNKF,LSTHCK,LSNPCL,LSNPLD,LSALNG,LSATH,
     *   LSCTCF,LSCHDF,LSLBDY,LSEVTF,
     *   LSATV,LSVC,LSVR,LSVL,LSPOR,LSSUMC,LSCNCN,LSCOLD,LSCAVG,
     *   LSPNWC,LSPNWR,LSPNWL,
     *   LSCI,LSCIN,LSCIR,LSCIRL,LSCIRH,LSIND,LSMRNO,LSMRNZ,
     *   LSRHS,LSVA,LSVAD,
     *   LSAP,LSPP,LSRA,LSRR,LSSS,LSXX,LSWW,LSWORK,LSRS,
     *   NCOL,NROW,NLAY,
     *   NSCOL,NSROW,NSLAY,NPMAX,NLIMBO,NEWPTS,IOUT,IOUTS,
     *   ICSTRT,ICONLY,NODISP,DIFFUS,NCINFL,NFACES,MOCTYPE,
     *   IUNIT(16),LSHFB,
     *   JUNIT(11),IDKTIM,
     *   LSDKZO,IDKZO,LSDKFO,IDKFO,
     *   LSDKZS,IDKZS,LSDKFS,IDKFS,
     *   JUNIT(10),LSDPCON,LSDPRAT,LSDPPOR,
     *   LSDPZO,IDPZO,LSDPFO,IDPFO)
C
CMOC----ALLOCATE SPACE FOR RECHARGE CONCENTRATION
C
      IF(IUNIT(8).GT.0.AND.IUNIT(15).GT.0) THEN
         CALL CRCH5AL(ISUM,LENX,LSCRCH,NSROW,NSCOL,
     *                JUNIT(1),IOUTS,IOUT,NRCHOP)
      ENDIF
C
CMOC----ALLOCATE SPACE FOR OBSERVATIONS
      IF(JUNIT(8).GT.0.AND.IUNIT(15).GT.0) THEN
         CALL OBS5AL(ISUM,LENX,LSOBSW,LSOBTE,NUMOBS,
     *                IOUT,IOUTS)
      ENDIF
C
C5------IF THE "X" ARRAY IS NOT BIG ENOUGH THEN STOP.
      IF(ISUM-1.GT.LENX) STOP
cellam
C       IF THE "MX" ARRAY IS NOT BIG ENOUGH THEN STOP.
      IF(NISUM-1.GT.LENMX) STOP
cellam
C
C6------READ AND PREPARE INFORMATION FOR ENTIRE SIMULATION.
      CALL BAS5RP(X(LCIBOU),X(LCHNEW),X(LCSTRT),X(LCHOLD),
     1       ISTRT,INBAS,HEADNG,NCOL,NROW,NLAY,VBVL,X(LCIOFL),
     2       IUNIT(12),IHEDFM,IDDNFM,IHEDUN,IDDNUN,IOUT,IPEROC,ITSOC,
     3       CHEDFM,CDDNFM,IBDOPT,IXSEC,LBHDSV,LBDDSV,IFREFM)
      IF(IUNIT(1).GT.0) CALL BCF5RP(X(LCIBOU),X(LCHNEW),X(LCSC1),
     1          X(LCHY),X(LCCR),X(LCCC),X(LCCV),X(LCDELR),
     2     X(LCDELC),X(LCBOT),X(LCTOP),X(LCSC2),X(LCTRPY),IUNIT(1),
     3     ISS,NCOL,NROW,NLAY,IOUT,X(LCWETD),IWDFLG,X(LCCVWD))
      IF(IUNIT(9).GT.0) CALL SIP5RP(NPARM,MXITER,ACCL,HCLOSE,X(LCW),
     1          IUNIT(9),IPCALC,IPRSIP,IOUT,IFREFM)
      IF(IUNIT(10).GT.0) CALL DE45RP(IUNIT(10),MXITER,NITER,ITMX,
     1            ACCL,HCLOSE,IFREQ,IPRD4,IOUT,MUTD4)
      IF(IUNIT(11).GT.0) CALL SOR5RP(MXITER,ACCL,HCLOSE,IUNIT(11),
     1         IPRSOR,IOUT,IFREFM)
      IF(IUNIT(13).GT.0) CALL PCG2RP(MXITER,ITER1,HCLOSE,RCLOSE,
     1         NPCOND,NBPOL,RELAX,IPRPCG,IUNIT(13),IOUT,MUTPCG,
     2         NITER,X(LCIT1),DAMP)
      IF(IUNIT(16).GT.0) CALL HFB1RP(X(LCCR),X(LCCC),X(LCDELR),           *HFB*
     1         X(LCDELC),X(LCHFBR),IUNIT(16),NCOL,NROW,NLAY,NODES,        *HFB*
     1         NHFB,IOUT)                                                 *HFB*
      IF(IUNIT(21).GT.0) CALL FHB1RP(X(LCIBOU),NROW,NCOL,NLAY,
     &          X(LCFLLC),X(LCBDTM),NBDTIM,X(LCFLRT),NFLW,NHED,
     &          X(LCHDLC),X(LCSBHD),IUNIT(21),IOUT,
     &          NFHBX1,NFHBX2,IFHBD3,IFHBD5)
C
CMOC----READ AND PREPARE INFORMATION FOR TRANSPORT
CMOC----CHECK FOR CONSISTENCY BETWEEN MODFLOW AND MOC PARAMETERS
      IF(IUNIT(15).GT.0) CALL MOC6CKRP(X(LCIBOU),X(LCDELR),X(LCDELC),
cellam
     *   X(LSCB),NTFACE,MX(LSLB),
     *   X(LSXFOR),X(LSXBAC),X(LSYFOR),X(LSYBAC),
cellam
     *   X(LCSC1),X(LCSC2),X(LSRF),
     *   X(LSCONC),X(LSCINT),X(LSCHBC),X(LSCINF),
     *   X(LSTHCK),X(LSALNG),X(LSATH),X(LSATV),
     *   X(LSPOR),X(LSPNWC),X(LSPNWR),X(LSPNWL),X(LSIGNP),
     *  FDTMTH,EPSSLV,IDIREC,NCXIT,MAXIT,
     *  SBVL,NEWPTS,NSCOL,NSROW,NSLAY,NODESS,
     *  NCOL,NROW,NLAY,IOUTS,INMOC,
     *  iunit,lcwell,nwelvl,mxwell,lcdrai,ndrnvl,mxdrn,
     *  lcrivr,nrivvl,mxrivr,lcbnds,nghbvl,mxbnd,
     *  lcievt,lsevtf,lscrch,lcirch,lcrech,LCHFBR,
cgzh bug fix  
     *  LCFLLC,LCBDFV,LCHDLC,LCBDHV,
cgzh
     *  ICSTRT,JRF,NODISP,NPNTVL,NPNTCL,NPNTDL,NPNTPL,
     *  IVELFM,ICONFM,IDSPFM,cnoflw,
     *  ICONLY,IFXHED,ISS,NCINFL,INTRPL,IDIM,NPTPND,MOCTYPE,
     *  X(LCCC),X(LCHY),X(LSCI),X(LSCIN),X(LSCIR),X(LSCIRH),
     *  X(LSCIRL),
     *  X(LSIND),X(LSMRNO),X(LSMRNZ),NBN,NRN, 
     *  X(LSOBSW),X(LSOBTE),NUMOBS,JUNIT,IOBSFL,NPNTOB,
     *  SRCDCY,DUM,DUM,DUM,NPER,DUM,TOTIM,
     *  X(LCHNEW),0.0,X(LCBUFF),
     *  X(LSDKZO),X(LSDKFO),X(LSDKZS),
     *  X(LSDKFS),IDKRF,IDKZO,IDKFO,IDKZS,IDKFS,
     *  x(LSDPCON),x(LSDPRAT),x(LSDPPOR),
     *  X(LSDPZO),X(LSDPFO),
     *  IDPZO,IDPFO,IDPPS)
C
CMOC----DEFINE HFB LOCATION ARRAY FOR MOC3D DISP CALC
c      IF (IUNIT(15).GT.0.AND.IUNIT(16).GT.0) 
c     *  CALL MOC6HFB(X(LSHFB),X(LCIBOU),
c     *      NCOL,NROW,NLAY,NSCOL,NSROW,NSLAY,IOUTS,    
c     *      X(LCHFBR),NHFB)
C
C7------SIMULATE EACH STRESS PERIOD.
      DO 300 KPER=1,NPER
      KKPER=KPER
C
C7A-----READ STRESS PERIOD TIMING INFORMATION.
      CALL BAS5ST(NSTP,DELT,TSMULT,PERTIM,KKPER,INBAS,IOUT,IFREFM)
C
C7B-----READ AND PREPARE INFORMATION FOR STRESS PERIOD.
      IF(IUNIT(2).GT.0) CALL WEL5RP(X(LCWELL),NWELLS,MXWELL,IUNIT(2),
     1             IOUT,NWELVL,IWELAL,IFREFM)
      IF(IUNIT(3).GT.0) CALL DRN5RP(X(LCDRAI),NDRAIN,MXDRN,IUNIT(3),
     1                 IOUT,NDRNVL,IDRNAL,IFREFM)
      IF(IUNIT(4).GT.0) CALL RIV5RP(X(LCRIVR),NRIVER,MXRIVR,IUNIT(4),
     1            IOUT,NRIVVL,IRIVAL,IFREFM)
      IF(IUNIT(5).GT.0) CALL EVT5RP(NEVTOP,X(LCIEVT),X(LCEVTR),
     1            X(LCEXDP),X(LCSURF),X(LCDELR),X(LCDELC),NCOL,NROW,
     1            IUNIT(5),IOUT,IFREFM)
      IF(IUNIT(7).GT.0) CALL GHB5RP(X(LCBNDS),NBOUND,MXBND,IUNIT(7),
     1              IOUT,NGHBVL,IGHBAL,IFREFM)
      IF(IUNIT(8).GT.0) CALL RCH5RP(NRCHOP,X(LCIRCH),X(LCRECH),
     1            X(LCDELR),X(LCDELC),NROW,NCOL,IUNIT(8),IOUT,IFREFM)
C
CMOC DP  READ DOUBLE POROSITY PARAMETERS FOR THIS STRESS PERIOD
      IF(IUNIT(15).GT.0.AND.JUNIT(10).GT.0) THEN
        IF(IDPTIM.EQ.1.AND.KKPER.NE.1)
     *    CALL DP6RP(X(LCIBOU),
     *     x(LSDPCON),x(LSDPRAT),x(LSDPPOR),
     *     X(LSDPZO),X(LSDPFO),
     *     IDPZO,IDPFO,
     *     NSCOL,NSROW,NSLAY,NODESS,
     *     NCOL,NROW,NLAY,
     *     JUNIT(10),KKPER,IOUT,IOUTS,MOCTYPE)
      END IF
C
CMOC DK  READ SIMPLE REACTION PARMS FOR THIS STRESS PERIOD
      IF(IUNIT(15).GT.0.AND.JUNIT(11).GT.0) THEN
        IF(IDKTIM.EQ.1.AND.KKPER.NE.1) 
     *    CALL DK6RP(X(LSRF),X(LSDKZO),X(LSDKFO),X(LSDKZS),
     *          X(LSDKFS),IDKRF,IDKZO,IDKFO,IDKZS,IDKFS,
     *          KKPER,IOUTS,JUNIT(11),
     *     NSCOL,NSROW,NSLAY,NODESS,MOCTYPE)
      END IF
C7C-----SIMULATE EACH TIME STEP.
      DO 200 KSTP=1,NSTP
      KKSTP=KSTP
C
C7C1----CALCULATE TIME-STEP LENGTH. SET HOLD=HNEW.
      CALL BAS5AD(DELT,TSMULT,TOTIM,PERTIM,X(LCHNEW),X(LCHOLD),KKSTP,
     1             NCOL,NROW,NLAY)
      IF(IUNIT(1).GT.0) CALL BCF5AD(X(LCIBOU),X(LCHOLD),X(LCBOT),
     1             X(LCWETD),IWDFLG,ISS,NCOL,NROW,NLAY)
      IF(IUNIT(21).GT.0) CALL FHB1AD(X(LCHNEW),X(LCHOLD),NCOL,NROW,NLAY,
     &          ISS,TOTIM,DELT,X(LCBDTM),NBDTIM,X(LCFLRT),
     &          X(LCBDFV),X(LCBDHV),NFLW,X(LCSBHD),X(LCHDLC),NHED,
     &          NFHBX1,NFHBX2,IFHBD3,IFHBD4,IFHBD5,IFHBSS)
cellam
C------COPY BEGINNING OF FLOW STEP IBOUND FOR ELLAM
         IF(moctype.eq.3.and.ISS.EQ.0.AND.IUNIT(15).GT.0)
     *          CALL SMOC6I(X(LCIBOU),X(LSIBOU),NODES)
cellam
c
C
C7C2----ITERATIVELY FORMULATE AND SOLVE THE EQUATIONS.
      DO 100 KITER=1,MXITER
      KKITER=KITER
C
C7C2A---FORMULATE THE FINITE-DIFFERENCE EQUATIONS.
      CALL BAS5FM(X(LCHCOF),X(LCRHS),NODES)
      IF(IUNIT(1).GT.0) CALL BCF5FM(X(LCHCOF),X(LCRHS),X(LCHOLD),
     1          X(LCSC1),X(LCHNEW),X(LCIBOU),X(LCCR),X(LCCC),X(LCCV),
     2          X(LCHY),X(LCTRPY),X(LCBOT),X(LCTOP),X(LCSC2),
     3          X(LCDELR),X(LCDELC),DELT,ISS,KKITER,KKSTP,KKPER,NCOL,
     4          NROW,NLAY,IOUT,X(LCWETD),IWDFLG,X(LCCVWD),WETFCT,
     5          IWETIT,IHDWET,HDRY,X(LCBUFF))
      IF(IUNIT(2).GT.0) CALL WEL5FM(NWELLS,MXWELL,X(LCRHS),X(LCWELL),
     1           X(LCIBOU),NCOL,NROW,NLAY,NWELVL)
      IF(IUNIT(3).GT.0) CALL DRN5FM(NDRAIN,MXDRN,X(LCDRAI),X(LCHNEW),
     1         X(LCHCOF),X(LCRHS),X(LCIBOU),NCOL,NROW,NLAY,NDRNVL)
      IF(IUNIT(4).GT.0) CALL RIV5FM(NRIVER,MXRIVR,X(LCRIVR),X(LCHNEW),
     1            X(LCHCOF),X(LCRHS),X(LCIBOU),NCOL,NROW,NLAY,NRIVVL)
      IF(IUNIT(5).GT.0) CALL EVT5FM(NEVTOP,X(LCIEVT),X(LCEVTR),
     1            X(LCEXDP),X(LCSURF),X(LCRHS),X(LCHCOF),X(LCIBOU),
     1            X(LCHNEW),NCOL,NROW,NLAY)
      IF(IUNIT(7).GT.0) CALL GHB5FM(NBOUND,MXBND,X(LCBNDS),X(LCHCOF),
     1            X(LCRHS),X(LCIBOU),NCOL,NROW,NLAY,NGHBVL)
      IF(IUNIT(8).GT.0) CALL RCH5FM(NRCHOP,X(LCIRCH),X(LCRECH),
     1            X(LCRHS),X(LCIBOU),NCOL,NROW,NLAY)
      IF(IUNIT(16).GT.0) CALL HFB1FM(X(LCHNEW),X(LCCR),X(LCCC),           *HFB*
     1          X(LCBOT),X(LCTOP),X(LCDELR),X(LCDELC),X(LCHFBR),          *HFB*
     2          NCOL,NROW,NLAY,NHFB)                                      *HFB*
      IF(IUNIT(21).GT.0) CALL FHB1FM(X(LCRHS),X(LCIBOU),X(LCFLLC),
     1          X(LCBDFV),NFLW,NCOL,NROW,NLAY,IFHBD4)
C
C7C2B---MAKE ONE CUT AT AN APPROXIMATE SOLUTION.
      IF(IUNIT(9).GT.0) CALL SIP5AP(X(LCHNEW),X(LCIBOU),X(LCCR),X(LCCC),
     1     X(LCCV),X(LCHCOF),X(LCRHS),X(LCEL),X(LCFL),X(LCGL),X(LCV),
     2     X(LCW),X(LCHDCG),X(LCLRCH),NPARM,KKITER,HCLOSE,ACCL,ICNVG,
     3     KKSTP,KKPER,IPCALC,IPRSIP,MXITER,NSTP,NCOL,NROW,NLAY,NODES,
     4     IOUT)
      IF(IUNIT(10).GT.0) CALL DE45AP(X(LCHNEW),X(LCIBOU),X(LCAU),
     1  X(LCAL),X(LCIUPP),X(LCIEQP),X(LCD4B),MXUP,MXLOW,MXEQ,MXBW,
     2  X(LCCR),X(LCCC),X(LCCV),X(LCHCOF),X(LCRHS),ACCL,KKITER,ITMX,
     3  MXITER,NITER,HCLOSE,IPRD4,ICNVG,NCOL,NROW,NLAY,IOUT,X(LCLRCH),
     4  X(LCHDCG),IFREQ,KKSTP,KKPER,DELT,NSTP,ID4DIR,ID4DIM,MUTD4)
      IF(IUNIT(11).GT.0) CALL SOR5AP(X(LCHNEW),X(LCIBOU),X(LCCR),
     1     X(LCCC),X(LCCV),X(LCHCOF),X(LCRHS),X(LCA),X(LCRES),X(LCIEQP),
     2     X(LCHDCG),X(LCLRCH),KKITER,HCLOSE,ACCL,ICNVG,KKSTP,KKPER,
     3     IPRSOR,MXITER,NSTP,NCOL,NROW,NLAY,NSLICE,MBW,IOUT)
cgzh      IF(IUNIT(37).GT.0) CALL BPCGAP(X(LCHNEW),X(LCIBOU),X(LCCR),
cgzh     1      X(LCCC),X(LCCV),X(LCHCOF),X(LCRHS),
cgzh     2      KKSTP,KKPER,ICNVG,MXITER,NCOL,NROW,
cgzh     3      NLAY,NODES,IUNIT(37),IOUT)
      IF(IUNIT(13).GT.0) CALL PCG2AP(X(LCHNEW),X(LCIBOU),X(LCCR),
     1      X(LCCC),X(LCCV),X(LCHCOF),X(LCRHS),X(LCV),X(LCSS),X(LCP),
     2      X(LCCD),X(LCHCHG),X(LCLHCH),X(LCRCHG),X(LCLRCH),KKITER,
     3      NITER,HCLOSE,RCLOSE,ICNVG,KKSTP,KKPER,IPRPCG,MXITER,ITER1,
     4      NPCOND,NBPOL,NSTP,NCOL,NROW,NLAY,NODES,RELAX,IOUT,MUTPCG,
     5      0,0,SN,SP,SR,X(LCIT1),DAMP)
C
C7C2C---IF CONVERGENCE CRITERION HAS BEEN MET STOP ITERATING.
      IF(ICNVG.EQ.1) GO TO 110
  100 CONTINUE
      KITER=MXITER
  110 CONTINUE
C
CMOC----INITIALIZE TRANSPORT TERMS BASED ON FIRST FLOW SOLUTION
C
cellam
      IF(moctype.eq.3.and.ISS.EQ.0.AND.IUNIT(15).GT.0) then
         lsave=lcibou
         lcibou=lsibou
      endif
cellam         
C
      IF (KSTP.EQ.1.AND.IUNIT(15).GT.0) CALL MOC6INIT(ICNVG,IUNIT,
     *   KSTP,KPER,X(LCHNEW),X(LCIBOU),X(LCHOLD),X(LCTOP),
     *   X(LSTHCK),X(LCBOT),NSCOL,NSROW,NSLAY,NCOL,NROW,NLAY,
     *   IOUTS,ISS,X(LSCONC),X(LSRF),X(LSPOR),SBVL,JRF,
     *   X(LSPC),X(LSPR),X(LSPL),X(LSPCON),X(LSPTID),X(LSNPCL),
     *   X(LSPNWC),X(LSPNWR),X(LSPNWL),X(LSLMBO),
     *   NEWPTS,NPMAX,NLIMBO,NP,MOCTYPE,
     *   JUNIT,KKPER,NPER,KKSTP,NSTP,DUM,NMOV,TIMV,NPNTPL,SUMTCH)
C
C7C3----DETERMINE WHICH OUTPUT IS NEEDED.
      CALL BAS5OC(NSTP,KKSTP,ICNVG,X(LCIOFL),NLAY,IBUDFL,ICBCFL,
     1   IHDDFL,IUNIT(12),IOUT,KKPER,IPEROC,ITSOC,IBDOPT,IXSEC,IFREFM)
C
C7C4----CALCULATE BUDGET TERMS. SAVE CELL-BY-CELL FLOW TERMS.
      MSUM=1
C7C4A---THE ORIGINAL BCF BUDGET MODULE HAS BEEN REPLACED BY THREE
C7C4A---SUBMODULES: SBCF5S, SBCF5F, AND SBCF5B .
      IF(IUNIT(1).GT.0) THEN
         CALL SBCF5S(VBNM,VBVL,MSUM,X(LCHNEW),X(LCIBOU),X(LCHOLD),
     1     X(LCSC1),X(LCTOP),X(LCSC2),DELT,ISS,NCOL,NROW,NLAY,KKSTP,
     2     KKPER,IBCFCB,ICBCFL,X(LCBUFF),IOUT,PERTIM,TOTIM)
C
CMOC----UPDATE FLUID STORAGE TERMS FOR TRANSPORT
cellam--if not ELLAM run
         IF(moctype.ne.3.and.ICNVG.EQ.1.AND.ISS.EQ.0.AND.IUNIT(15).GT.0)
     *     CALL SMOC5UP(X(LCHNEW),X(LCIBOU),X(LCHOLD),X(LCTOP),
     *     X(LCSC1),X(LCSC2),X(LSTHCK),X(LSPOR),X(LCBUFF),DELT,
     *     NSCOL,NSROW,NSLAY,NCOL,NROW,NLAY,IOUTS,
cellam
     *     MOCTYPE,X(LCDELR),X(LCDELC))
cellam
C
         CALL SBCF5F(VBNM,VBVL,MSUM,X(LCHNEW),X(LCIBOU),X(LCCR),
     1     X(LCCC),X(LCCV),X(LCTOP),DELT,NCOL,NROW,NLAY,KKSTP,KKPER,
     2     IBCFCB,X(LCBUFF),IOUT,ICBCFL,PERTIM,TOTIM,ICHFLG)
C
CMOC----FOR TRANSPORT, INITIALIZE SINK/SOURCE ARRAYS
CMOC----AND COMPUTE TERMS AT FIXED HEADS
         IF(ICNVG.EQ.1.AND.IUNIT(15).GT.0.AND.IUNIT(21).GT.0)
     *   CALL CFHBH1FM(X(LCHDLC),X(LCBDHV),NHED,NFHBX2,IFHBHC,
     *      X(LSCHBC),NCOL,NROW,NLAY,KSTP,KPER,
     *      NSCOL,NSROW,NSLAY,IOUTS,ICONLY)
         IF(ICNVG.EQ.1.AND.IUNIT(15).GT.0)
     *   CALL SRC5FM(X(LSEVTF),X(LSCHDF),
     *      X(LCBUFF),X(LCIBOU),X(LSSRCF),X(LSSRCS),X(LSSNKF),X(LSCHBC),
     *      SRCDCY,NCOL,NROW,NLAY,NSCOL,NSROW,NSLAY,IOUTS,IFXHED,ICONLY)
C
         IBDRET=0
         IF(ICNVG.EQ.1.AND.IUNIT(15).GT.0) THEN
            IBDRET=1
         END IF
C
C  CALCULATE FLOW BETWEEN EACH CELL IN ALL DIRECTIONS
         DO 155 IDIR=1,3
         CALL SBCF5B(X(LCHNEW),X(LCIBOU),X(LCCR),X(LCCC),X(LCCV),
     1      X(LCTOP),NCOL,NROW,NLAY,KKSTP,KKPER,IBCFCB,X(LCBUFF),
     2      IOUT,ICBCFL,DELT,PERTIM,TOTIM,IDIR,IBDRET,ICHFLG,
     3      IC1,IC2,IR1,IR2,IL1,IL2)
          IF(ICNVG.EQ.1.AND.IUNIT(15).GT.0) THEN
C
CMOC----CALCULATE FLOWS ACROSS BOUNDARIES OF SUBGRID
CMOC----CALCULATE AND PRINT VELOCITIES
           CALL MOC6VELO(X(LSCTCF),X(LCBUFF),X(LSLBDY),
     *      IDIR,ID,NCOL,NROW,NLAY,IC1,IC2,IR1,IR2,IL1,
     *      IL2,NFACES,NCINFL,
     *      X(LSTHCK),X(LSPOR),X(LSRF),X(LSVC),X(LSVR),X(LSVL),
     *      X(LCIBOU),NSCOL,NSROW,NSLAY,IOUTS,NTIMV,
     *      VCMAX,VRMAX,VLMAX,TLMIN,
     *      MAXVCJ,MAXVCI,MAXVCK,MAXVRJ,MAXVRI,MAXVRK,
     *      MAXVLJ,MAXVLI,MAXVLK,
     *      DELT,KKSTP,KKPER,NSTP,NPER,TIMV,TOTIM,ITCD,
     *      NPNTVL,IVELFM,JUNIT,
cellam
     *      MOCTYPE,TCMIN,TRMIN,X(LCDELR),X(LCDELC))
cellam
C
          END IF
155      CONTINUE
C
CMOC----CALCULATE AND PRINT DISPERSION COEFFICIENTS
         IF(ICNVG.EQ.1.AND.IUNIT(15).GT.0) THEN
            IF(NODISP.NE.1.OR.DIFFUS.NE.0.0) THEN
              CALL MOC6DISP(X(LSDCC),
     *          X(LSDCR),X(LSDCL),X(LSDRR),X(LSDRC),X(LSDRL),X(LSDLL),
     *          X(LSDLC),X(LSDLR),X(LSTHCK),X(LSALNG),X(LSATH),X(LSATV),
     *          X(LSPOR),X(LSRF),X(LSVC),X(LSVR),X(LSVL),X(LCIBOU),
     *          MOCTYPE,DELT,NCOL,NROW,NLAY,NSCOL,NSROW,NSLAY,
     *          IOUTS,NTIMD,TIMDC,NODISP,DIFFUS,TIMV,
     *          KKSTP,NSTP,KKPER,NPER,IDSPFM,NPNTDL,
cellam          
     *          X(LCDELR),X(LCDELC),
cellam
cgzh
     *          IUNIT(16),X(LCHFBR),NHFB)
cgzh
            END IF
         END IF
      END IF
      IF(IUNIT(2).GT.0) CALL WEL5BD(NWELLS,MXWELL,VBNM,VBVL,MSUM,
     1     X(LCWELL),X(LCIBOU),DELT,NCOL,NROW,NLAY,KKSTP,KKPER,IWELCB,
     1     ICBCFL,X(LCBUFF),IOUT,PERTIM,TOTIM,NWELVL,IWELAL)
C
CMOC----SAVE SOURCE/SINK FLUXES FOR MASS-BALANCE CALCULATION
CMOC----FLUX CARRIED IN PACKAGE ARRAY WITH "CBCALLOCATE" OPTION
C
CMOC----WELLS
      IF(IUNIT(2).GT.0.AND.ICNVG.EQ.1.AND.IUNIT(15).GT.0)
     * CALL CWEL5FM(NWELLS,MXWELL,X(LCWELL),X(LCIBOU),NCOL,NROW,NLAY,
     *   KKSTP,KKPER,X(LSSRCF),X(LSSRCS),X(LSSNKF),
     *   NSCOL,NSROW,NSLAY,IOUTS,NWELVL,IWELAL,IWELLC,ICONLY)
      IF(IUNIT(3).GT.0) CALL DRN5BD(NDRAIN,MXDRN,VBNM,VBVL,MSUM,
     1     X(LCDRAI),DELT,X(LCHNEW),NCOL,NROW,NLAY,X(LCIBOU),KKSTP,
     2     KKPER,IDRNCB,ICBCFL,X(LCBUFF),IOUT,PERTIM,TOTIM,NDRNVL,
     3     IDRNAL)
CMOC----DRAINS
      IF(IUNIT(3).GT.0.AND.ICNVG.EQ.1.AND.IUNIT(15).GT.0)
     * CALL CDRN5FM(NDRAIN,MXDRN,X(LCDRAI),X(LCIBOU),NCOL,NROW,NLAY,
     *   X(LSSNKF),NSCOL,NSROW,NSLAY,
     *   IOUTS,NDRNVL,IDRNAL,ICONLY,KKSTP)
      IF(IUNIT(4).GT.0) CALL RIV5BD(NRIVER,MXRIVR,X(LCRIVR),X(LCIBOU),
     1     X(LCHNEW),NCOL,NROW,NLAY,DELT,VBVL,VBNM,MSUM,KKSTP,KKPER,
     2     IRIVCB,ICBCFL,X(LCBUFF),IOUT,PERTIM,TOTIM,NRIVVL,IRIVAL)
CMOC----RIVERS
      IF(IUNIT(4).GT.0.AND.ICNVG.EQ.1.AND.IUNIT(15).GT.0)
     * CALL CRIV5FM(NRIVER,MXRIVR,X(LCRIVR),X(LCIBOU),NCOL,NROW,NLAY,
     *   KKSTP,KKPER,X(LSSRCF),X(LSSRCS),X(LSSNKF),
     *   NSCOL,NSROW,NSLAY,IOUTS,NRIVVL,IRIVAL,IRIVRC,ICONLY)
      IF(IUNIT(5).GT.0) CALL EVT5BD(NEVTOP,X(LCIEVT),X(LCEVTR),
     1     X(LCEXDP),X(LCSURF),X(LCIBOU),X(LCHNEW),NCOL,NROW,NLAY,
     2     DELT,VBVL,VBNM,MSUM,KKSTP,KKPER,IEVTCB,ICBCFL,X(LCBUFF),IOUT,
     3     PERTIM,TOTIM)
CMOC----EVAPOTRANSPIRATION
      IF(IUNIT(5).GT.0.AND.ICNVG.EQ.1.AND.IUNIT(15).GT.0)
     * CALL CEVT5FM(NEVTOP,X(LCBUFF),X(LSEVTF),X(LCIEVT),X(LCIBOU),
     *     NCOL,NROW,NLAY,
     *     NSCOL,NSROW,NSLAY,IOUTS,ICONLY)
      IF(IUNIT(7).GT.0) CALL GHB5BD(NBOUND,MXBND,VBNM,VBVL,MSUM,
     1     X(LCBNDS),DELT,X(LCHNEW),NCOL,NROW,NLAY,X(LCIBOU),KKSTP,
     2     KKPER,IGHBCB,ICBCFL,X(LCBUFF),IOUT,PERTIM,TOTIM,NGHBVL,
     3     IGHBAL)
CMOC----GENERAL HEAD BOUNDARIES
      IF(IUNIT(7).GT.0.AND.ICNVG.EQ.1.AND.IUNIT(15).GT.0)
     * CALL CGHB5FM(NBOUND,MXBND,X(LCBNDS),X(LCIBOU),NCOL,NROW,NLAY,
     *   KKSTP,KKPER,X(LSSRCF),X(LSSRCS),X(LSSNKF),
     *   NSCOL,NSROW,NSLAY,IOUTS,NGHBVL,IGHBAL,IBNDSC,ICONLY)
      IF(IUNIT(8).GT.0) CALL RCH5BD(NRCHOP,X(LCIRCH),X(LCRECH),
     1     X(LCIBOU),NROW,NCOL,NLAY,DELT,VBVL,VBNM,MSUM,KKSTP,KKPER,
     2     IRCHCB,ICBCFL,X(LCBUFF),IOUT,PERTIM,TOTIM)
CMOC----RECHARGE
      IF(IUNIT(8).GT.0.AND.ICNVG.EQ.1.AND.IUNIT(15).GT.0)
     * CALL CRCH5FM(NRCHOP,X(LCIRCH),X(LSCRCH),X(LCBUFF),
     *   X(LCIBOU),X(LSSRCF),X(LSSRCS),X(LSSNKF),NCOL,NROW,NLAY,
     *   NSCOL,NSROW,NSLAY,IOUTS,JUNIT(1),KKSTP,ICONLY)
      IF(IUNIT(21).GT.0) CALL FHB1BD(X(LCFLLC),X(LCBDFV),NFLW,
     1     VBNM,VBVL,MSUM,X(LCIBOU),DELT,NCOL,NROW,NLAY,KKSTP,KKPER,
     2     IFHBCB,ICBCFL,X(LCBUFF),IOUT,IFHBD4)
CMOC----FLOW-HEAD BOUNDARIES
      IF(IUNIT(21).GT.0.AND.ICNVG.EQ.1.AND.IUNIT(15).GT.0)
     * CALL CFHBF1FM(X(LCFLLC),X(LCBDFV),NFLW,IFHBD4,NFHBX1,IFHBFC,
     *   X(LCIBOU),NCOL,NROW,NLAY,KSTP,KPER,
     *   X(LSSRCF),X(LSSRCS),X(LSSNKF),
     *   NSCOL,NSROW,NSLAY,IOUTS,ICONLY)
C
C7C5---PRINT AND OR SAVE HEADS AND DRAWDOWNS. PRINT OVERALL BUDGET.
      CALL BAS5OT(X(LCHNEW),X(LCSTRT),ISTRT,X(LCBUFF),X(LCIOFL),
     1     MSUM,X(LCIBOU),VBNM,VBVL,KKSTP,KKPER,DELT,PERTIM,TOTIM,
     2     ITMUNI,NCOL,NROW,NLAY,ICNVG,IHDDFL,IBUDFL,IHEDFM,IHEDUN,
     3     IDDNFM,IDDNUN,IOUT,CHEDFM,CDDNFM,IXSEC,LBHDSV,LBDDSV)
C
C7C6----IF ITERATION FAILED TO CONVERGE THEN STOP.
      IF(ICNVG.EQ.0) STOP
C
CMOC----SKIP IF TRANSPORT OPTION NOT SELECTED ("CONC" for MOC3D)
      IF(IUNIT(15).LE.0) GO TO 190
C
CMOC DP  COMPUTE TIME STEP STABILITY FOR DOUBLE POROSITY (EXPLICIT ONLY)
      IF(JUNIT(10).GT.0) THEN
        IF(MOCTYPE.EQ.1) THEN
          CALL DP6ST(X(LCIBOU),X(LSRF),X(LSPOR),
     1      TIMDP,JMAXDP,IMAXDP,KMAXDP,
     2      X(LSDPRAT),X(LSDPPOR),
     3      NCOL,NROW,NLAY,NSCOL,NSROW,NSLAY,IOUTS)
        END IF
      END IF
CMOC----COMPUTE TRANSPORT TIME STEP (TIMV) AND NUMBER OF MOVES
CMOC----REQUIRED FOR THIS FLOW TIME STEP (NMOV), INITIALIZE TRANSPORT
CMOC----TIME (SUMTCH), ZERO CNCNC BEFORE MOVE LOOP
      CALL MOC6ST(X(LSSRCF),X(LSSNKF),X(LSRF),X(LSPOR),X(LSTHCK),
     *  VCMAX,VRMAX,VLMAX,TLMIN,TIMV,NTIMV,ITCD,
     *  MAXVCJ,MAXVCI,MAXVCK,MAXVRJ,MAXVRI,MAXVRK,MAXVLJ,MAXVLI,MAXVLK,
     *  TIMDC,NTIMD,
     *  NMOV,SUMTCH,NSCOL,NSROW,NSLAY,IOUTS,KSTP,KPER,
     *  TOTIM,DELT,NODISP,DIFFUS,DECAY,ICONLY,MOCTYPE,
     *  JUNIT(10),TIMDP,JMAXDP,IMAXDP,KMAXDP)
C
C Load and assemble--implicit MOC3D transport solver
      IF(MOCTYPE.EQ.2)
     *  CALL MOC5LDAS(
     *     X(LCIBOU),NCOL,NROW,NLAY,
     *     X(LSDCC),X(LSDCR),X(LSDCL),X(LSDRR),X(LSDRC),X(LSDRL),
     *     X(LSDLL),X(LSDLC),X(LSDLR),
     *     NSCOL,NSROW,NSLAY,
     *     X(LSTHCK),X(LSPOR),X(LSRF),TIMV,
     *     X(LSVA),X(LSVAD),X(LSRHS),X(LSCIN),X(LSMRNO),
     *     FDTMTH,NRN,NODESS)
C
CMOC----PERFORM EACH MOVE
      DO 188 IMOV=1,NMOV
           NRFLG=0
         IIMOV=IMOV
      CALL MOC6MVOT(X(LSSRCF),X(LSSNKF),X(LSRF),X(LSPOR),X(LSTHCK),
     *  VCMAX,VRMAX,VLMAX,TLMIN,TIMV,NTIMV,ITCD,
     *  MAXVCJ,MAXVCI,MAXVCK,MAXVRJ,MAXVRI,MAXVRK,MAXVLJ,MAXVLI,MAXVLK,
     *  TIMDC,NTIMD,NMOV,SUMTCH,NSCOL,NSROW,NSLAY,IOUTS,
     *  TOTIM,DELT,NODISP,DIFFUS,DECAY,ICONLY,MOCTYPE,
     *  IIMOV,X(LSCONC),X(LSCOLD),X(LSNPCL),X(LSNPLD),X(LSHFB),
     *  INTRPL,X(LSPC),X(LSPR),X(LSPL),X(LSPCON),X(LSPTID),
     *  X(LSVC),X(LSVR),X(LSVL),X(LSCINF),X(LCIBOU),X(LCDELR),X(LCDELC),
     *  X(LSSUMC),X(LSLMBO),X(LSPNWC),X(LSPNWR),X(LSPNWL),X(LSIGNP),
     *  NEWPTS,NCOL,NROW,NLAY,NPMAX,NLIMBO,NP,NCINFL,IABOVE,IBELOW,
     *  X(LSCNCN),X(LSSRCS),X(LSEVTF),X(LCIEVT),SRCDCY,NEVTOP,
     *  KKSTP,KKPER,SNKRAT,X(LSRS),FDTMTH,X(LSCAVG),
     *  X(LSDCC),X(LSDCR),X(LSDCL),X(LSDRR),X(LSDRC),X(LSDRL),
     *  X(LSDLL),X(LSDLC),X(LSDLR),X(LSVA),X(LSVAD),X(LSRHS),X(LSCIN),
     *  X(LSMRNO),X(LSAP),X(LSPP),X(LSRA),X(LSRR),X(LSSS),
     *  X(LSXX),X(LSWW),X(LSWORK),X(LSCI),X(LSCIR),X(LSCIRH),X(LSCIRL),
     *  NCXIT,EPSSLV,MAXIT,NBN,NRN,
     *  X(LSCHBC),X(LSCHDF),X(LSCTCF),X(LSLBDY),
     *  X(LCRECH),X(LSCRCH),X(LCIRCH),NRCHOP,
     *  X(LCWELL),MXWELL,NWELLS,NWELVL,IWELLC,
     *  X(LCRIVR),MXRIVR,NRIVER,NRIVVL,IRIVRC,
     *  X(LCDRAI),MXDRN,NDRAIN,NDRNVL,
     *  X(LCBNDS),MXBND,NBOUND,NGHBVL,IBNDSC,
     *  X(LCFLLC),X(LCBDFV),NFLW,IFHBD4,IFHBFC,
     *  X(LCHDLC),X(LCBDHV),NHED,NFHBX2,IFHBHC,
     *  IUNIT,JRF,SBVL,NFACES,NSTP,NPER,JUNIT,PERTIM,
     *  NPNTCL,ICONFM,X(LCHNEW),X(LSOBSW),X(LSOBTE),
     *  NUMOBS,IOBSFL,NPNTPL,NODESS,
     *  NPNTOB,X(LSDKZO),X(LSDKFO),X(LSDKZS),
     *  X(LSDKFS),IDKZO,IDKFO,IDKZS,IDKFS,
     *  AGER8,
     *  x(LSDPCON),x(LSDPRAT),x(LSDPPOR),
     *  X(LSDPZO),X(LSDPFO),
     *  IDPZO,IDPFO,IDPPS,
cellam
     *  X(LSRHSE),X(LSRHSO),NTFACE,X(LSDO),X(LSDIST),X(LSDIDS),
     *  MX(LSIDB),MX(LSLB),
     *  X(LSXFOR),X(LSXBAC),X(LSYFOR),X(LSYBAC),X(LSCB),MX(LSNZIN),
     *  MX(LSNONU),X(LSSAV),MX(LSIACT),X(LSCFOR),X(LSRFOR),X(LSTFOR),
     *  X(LSCONL),X(LSVOL),X(LSRW),LENW,MX(LSJA),MX(LSIA),X(LSA),IDIM,
     *  NELT,NODES,MX(LSJAS),MX(LSIAS),X(LSAS),NELTS,LENIW,MX(LSIW),
     *  NRFLG)
cellam
C
CMOC----END OF MOVE LOOP
  188 CONTINUE
C
C
CMOC----UPDATE FLUID STORAGE TERMS FOR TRANSPORT
cellam--if ELLAM run
         IF(moctype.eq.3.and.ISS.EQ.0) THEN
           call ellflow(X(LCHNEW),X(LCIBOU),X(LCHOLD),
     1     X(LCSC1),X(LCTOP),X(LCSC2),DELT,NCOL,NROW,NLAY,
     2     X(LCBUFF),IOUT)
c           CALL SMOC6UP(X(LCHNEW),X(LCIBOU),X(LCHOLD),X(LCTOP),
           CALL SMOC6UP(X(LCHNEW),X(LCHOLD),X(LCTOP),
     *     X(LCSC1),X(LCSC2),X(LSTHCK),X(LSPOR),X(LCBUFF),DELT,
     *     NSCOL,NSROW,NSLAY,NCOL,NROW,NLAY,IOUTS,
     *     MOCTYPE,X(LCDELR),X(LCDELC),
     *     X(LSRW),X(LSRHSE),X(LSCONC),
cch
     *     x(lcibou),x(lsibou),
     *     HDRY,X(LCCR),X(LCCC),X(LCCV),KKPER,KKSTP,KKITER,IOUT)
cch       if(moctype.eq.3) then
C   initialize DIAGST matrix
         CALL SMOC5Z(X(LSDIST),27,NODESS,1,0.D0)
         imov=1
         NRFLG=1
C   calculate matrix entries using updated thickness, porosity
         call BDYINT(IMOV,NSCOL,NSROW,NSLAY,NCOL,NROW,NLAY,
     *      JRF,X(LCDELR),X(LCDELC),X(LSTHCK),
     *      X(LCIBOU),NODESS,X(LSCOLD),X(LSCB),X(LSRHSE),
     *      X(LSRHSO),NTFACE,X(LSVC),X(LSVR),X(LSVL),X(LSRF),
     *      X(LSPOR),TIMV,VCMAX,VRMAX,VLMAX,IOUTS,X(LSDIST),MX(LSLB),
     *      MX(LSNZIN),NFACES,MX(LSNONU),X(LSSAV),X(LSXFOR),X(LSXBAC),
     *      X(LSYFOR),X(LSYBAC),X(LSCINF),NCINFL,IABOVE,IBELOW,DECAY,
     *      IDIM,NRFLG)
         CALL ELLLHI(IMOV,NSCOL,NSROW,NSLAY,
     *      NODESS,NCOL,NROW,NLAY,X(LCIBOU),X(LSCOLD),
     *      X(LSDIST),X(LSXFOR),
     *      X(LSYFOR),X(LSTHCK),MX(LSIACT),X(LSCFOR),X(LSRFOR),
     *      X(LSTFOR),
     *      X(LSCONL),X(LSVOL),X(LCDELR),X(LCDELC),
     *      X(LSRHSE),X(LSRHSO),MX(LSLB),MX(LSNZIN),
     *      NTFACE,X(LSVC),X(LSVR),X(LSVL),X(LSRF),X(LSPOR),TIMV,
     *      VCMAX,VRMAX,VLMAX,IOUTS,
     *      X(LSXBAC),X(LSYBAC),DECAY,NRFLG)
C   find interior concentrations giving correct mass with updated
C   thickness, porosity
C   - convert matrix
C   - solve for concentration using mass/por in cell from last
C     transport time step
C     (stored in RHS from rwork (temp) by smocup)
       CALL SMOC5I(JRF,NODES,NODESS,NSCOL,NSROW,NSLAY,
     *    NCOL,NROW,NLAY,MX(LSNZIN),X(LSRW),MX(LSJAS),MX(LSIAS),
     *    X(LSAS),NELTS,X(LSRW),LENW,LENIW,KKSTP,KKPER,
     *    X(LSCOLD),X(LSRF),X(LSDIST),X(LSPOR),X(LCIBOU),SBVL,
     *    NRFLG,
cch
     *    X(LSTHCK))
      CALL ELLSLV(X(LCIBOU),X(LSDIST),X(LSDIDS),
     *            X(LSRHSE),X(LSCONC),NCOL,NROW,
     *            NLAY,NODES,NODESS,NSCOL,NSROW,NSLAY,
     *            X(LSRW),LENW,MX(LSIW),LENIW,MX(LSIAS),
     *            MX(LSJAS),
     *            X(LSAS),NELTS,
     *            imov,nmov,
     *            X(LCDELR),X(LCDELC),x(lsthck),IOUTS)
C   find outflow conc using new thickness, porosity
      CALL ELLOUT(X(LSCB),X(LSDO),X(LSRHSO),MX(LSNONU),
     *      X(LSSAV),MX(LSIDB),
     *      X(LSRHSE),X(LSRF),X(LCIBOU),NSCOL,NSROW,NSLAY,NTFACE,
     *      NODESS,
     *      JRF,NFACES,X(LSPOR),x(lscold),NCOL,NROW,NLAY,NRFLG)
C   move new nodal conc to noflow nodes
      CALL ELNUPD(X(LSCB),MX(LSLB),
     *            X(LSCONC),NODESS,NTFACE,NFACES,
     *            NSCOL,NSROW,NSLAY)
         ENDIF
C
C
CMOC----SKIP TO HERE IF NO TRANSPORT
  190 CONTINUE
C
  200 CONTINUE
  300 CONTINUE
C
C8------END OF SIMULATION
      IF(IBATCH.GT.0) THEN
         WRITE(IBOUTS,*) ' Normal termination of simulation.'
         DO 400 I=1,IBOUTS-1
            INQUIRE(UNIT=I,OPENED=EXISTS)
            IF(EXISTS) CLOSE(I)
  400    CONTINUE
         GO TO 50
      END IF
  500 STOP
C
      END
